var express = require('express');
var cookieParser = require('cookie-parser');
var expressSession = require('express-session');
var MongoStore = require('connect-mongo')(expressSession);
var routes = require('./routes/index'); // ./routes/index.js -- file
var config = require('./config/settings.json');		
var auth	= require('./lib/auth-passport');  
var system  = require('./lib/dle-system');
var compress = require('compression');
var packageJson = require('./package.json');
//https libs
var http = require("http");
var https = require("https");
var fs = require('fs');
var helmet = require('helmet');
var csp = require('helmet')(helmet-csp);


//configuration
var app = express();
app.set('view engine', 'ejs');	//use ejs - embedded javascript
app.use(compress());    //compress all requests

//Set https security headers
app.use(helmet());

//DLEA-1251
app.use(helmet.xssFilter()); 

//DLEA-1252
var maxHttpsAge = 31536000; 
app.use(helmet.hsts({ 
	maxAge: maxHttpsAge,
	force: true
}));

//DLEA-1253
app.use(helmet.csp({
  // Specify directives as normal. 
  directives: {
    defaultSrc: ["'self'"],
    scriptSrc: ["'self'", "'unsafe-inline'", "'unsafe-eval'"], //inline required by EJS and eval by underscore
    styleSrc: ["'self'"],
    imgSrc: ["'self'"],
    //removed sandbox to allow safari/chrome to open new browser tab
    //sandbox: ['allow-scripts', 'allow-same-origin'], //safari required for backbone- allow-scripts+allow-same-origin
    objectSrc: [], // An empty array allows nothing through 
    //frameAncestors: ["'none'"], //unknown by safari
  },
 
  // Set to true if you only want browsers to report errors, not block them 
  reportOnly: false,
 
  // Set to true if you want to blindly set all headers: Content-Security-Policy, 
  // X-WebKit-CSP, and X-Content-Security-Policy. 
  setAllHeaders: false,
 
  // Set to true if you want to disable CSP on Android where it can be buggy. 
  disableAndroid: false,
 
  // Set to false if you want to completely disable any user-agent sniffing. 
  // This may make the headers less compatible but it will be much faster. 
  // This defaults to `true`. 
  browserSniff: false
}));

//DLEA-1254
app.use(helmet.nosniff());


//validate that package.json is populated
if (!(packageJson.config.appBuildDate.length > 0)) {
	system.logger.fatal('Application Build Date is Empty!');
};
if (!(packageJson.config.dbConnInfo.name.length > 0)) {
	system.logger.fatal('DB Name is Empty!');
};
if (!(packageJson.config.dbConnInfo.host.length > 0)) {
	system.logger.fatal('DB Host Name is Empty!');
};
if (!(packageJson.config.dbConnInfo.username.length > 0)) {
	system.logger.warn('DB Host UserName is Empty!');
};
if (!(packageJson.config.dbConnInfo.password.length > 0)) {
	system.logger.warn('DB Host Password is Empty!');
};
if (!(packageJson.config.dbConnInfo.port > 0)) {
	system.logger.fatal('DB Port is Empty!');
};
if (!(packageJson.config.sessionSecret.length > 0)) {
	system.logger.fatal('Session Secret is Empty!');
};

//session config
try {
if (config.nodeStartUp.protocol==='server') {
	app.set('trust proxy', 1); //trust proxy for secure cookie
	if ((packageJson.config.dbConnInfo.username.length > 0) &&
		(packageJson.config.dbConnInfo.password.length > 0) ) {			
		app.use(cookieParser());
		app.set('trust proxy', 1); // trust first proxy
		app.use(expressSession({ 
		    secret: packageJson.config.sessionSecret,
		    resave: false, // don't save session if unmodified
			saveUninitialized: false, // don't create session until something stored
			secure: true,
		    store: new MongoStore({
		    	db: packageJson.config.dbConnInfo.name,
		        host: packageJson.config.dbConnInfo.host,
		        port: packageJson.config.dbConnInfo.port,
		        username: packageJson.config.dbConnInfo.username,
	        	password: packageJson.config.dbConnInfo.password
			}),
			proxy: true,
			cookie: { secure: true }
		}));
		system.logger.warn('Connected to Server Mongo w/credentials');
	//anonymous Mongo connection	
	} else {
		app.use(cookieParser());
		app.set('trust proxy', 1); // trust first proxy
		app.use(expressSession({ 
		    secret: packageJson.config.sessionSecret,
		    resave: false, // don't save session if unmodified
			saveUninitialized: false, // don't create session until something stored
			secure: true,
		    store: new MongoStore({
		    	db: packageJson.config.dbConnInfo.name,
		        host: packageJson.config.dbConnInfo.host,
		        port: packageJson.config.dbConnInfo.port
			}),
			proxy: true,
			cookie: { secure: true }
		}));
		system.logger.warn('Connected to Server Mongo anonymously');	
	}
} else {
	app.use(expressSession({
		resave: false, // don't save session if unmodified
		saveUninitialized: false, // don't create session until something stored
		secure: true,
		secret: packageJson.config.sessionSecret
	}));
	system.logger.warn('Connected to Local Session');
}
} catch(err) {
	system.logger.fatal('Failure with session: '+err);
};

process.on('uncaughtException', function(err) {
  system.logger.fatal('Caught exception: '+err);
  system.logger.fatal(err.stack);
 
  //interrogate the error, if it's something you can recover from, let it be.
  //if the exception is fatal, exit with prejudice
  setTimeout(process.exit.bind(process, 666), 1000); //exit in a second
});

//namespacing to protect module variables
//access like this alert(dle.embedded);
//other apps can call it the sameway but need to 
//be later in the js load order
var dle = (function () {

	var route = config.nodeStartUp.protocol==='server'?'DSLogonEducation':'dle';

	return {
		route: route
	};
	
} )(); //end of namespace
system.logger.warn('app route: '+dle.route);

//content passed to all ejs pages -- Added locally to every page/template/partial
app.locals.access = '0';
app.locals.appVersion = packageJson.version;
app.locals.appUpdatedDate = packageJson.config.appBuildDate;
app.locals.errors = '';
app.locals.helpPage = '';
app.locals.name = '';
app.locals.launchPadLink = config.launchPadLink;
app.locals.route = dle.route;
system.logger.warn('route: ' + app.locals.route);
 
//Fixed passport.initialize() middleware not in use
app.use(express.static(__dirname + '/public'));

//Fixes CORS cross-origin ressource sharing error
 app.all('*', function (request, response, next) {
     'use strict';
     // log.debug('URL:' + request.originalUrl);

     response.header('Access-Control-Allow-Origin', '*');
     response.header('Access-Control-Allow-Methods', 'PUT, GET, POST, DELETE, OPTIONS');
     response.header('Access-Control-Allow-Headers', 'Content-Type');
     next();
 });

app.use(auth.passport.initialize());
//app.use(auth.passport.session()); //not sure its needed
 
//Serve static content to SPA
app.use('/dle', express.static(__dirname + '/public'));
app.use('/dle/help', express.static(__dirname + '/public'));
app.use('/dle/success/token', express.static(__dirname + '/public'));
app.use('/dle/iam/auth', express.static(__dirname + '/public'));

//testing process to capture oAuth exceptions
//app.use(errorHandler);  //haven't seen it setting

system.logger.warn('__dirname: '+__dirname);

app.get('/dle', routes.dle);
app.get('/dle/index.html', routes.dle); 
app.get('/dle/help/:page', routes.helpPage); 
app.get('/dle/failure', routes.dleFailure); 
app.get('/dle/iam/success/token', routes.dleSuccessToken); 
app.get('/dle/iam/auth/logout', routes.dleAuthLogout); 
app.get('/dle/auth/provider', routes.authProvider);  	//oAuth entry point from the client for full screen 

// The OAuth 2.0 provider has redirected the user back to the application.
// Finish the authentication process by attempting to obtain an access
// token.  If authorization was granted, the user will be logged in.
// Otherwise, authentication has failed.
app.get('/dle/auth/provider/callback',  
  	auth.passport.authenticate('provider', { successRedirect: '/'+dle.route+'/iam/success/token',
  	failureRedirect: '/'+dle.route+'/failure'}));

app.use( function(err, req, res, next){   			//captures errors and exceptions
	system.logger.error('node errors and exceptions: '+err.message);
	system.logger.debug('err.stack: '+err.stack);
	auth.errors = [['7',err.message]];
	res.redirect("/"+dle.route+"/failure");  //res.render doesn't properly render like no access to public
  	//next(err);
});

//we may need to remove this for CRISP
/** 
app.get('/*', function(req, res) {
	system.logger.warn('server hit with this invalid url path: '+req.url);
 	res.send('if all else fails, we hit this page');
});
**/

//Setup node listener -- events.js:72 indicates that server port is already used
system.logger.warn('protocol (local/server): '+config.nodeStartUp.protocol);

if (config.nodeStartUp.protocol==='local') {
	http.createServer(app).listen(config.nodeStartUp.localHttpPort);
	system.logger.warn('listening on http port: '+config.nodeStartUp.localHttpPort);

	var options = {
		key: fs.readFileSync(packageJson.config.keyFileName),
		cert: fs.readFileSync(packageJson.config.certFileName),
		//ciphers from the iojs -- https://certsimple.com/blog/a-plus-node-js-ssl
		ciphers: [
		    "ECDHE-RSA-AES256-SHA384",
		    "DHE-RSA-AES256-SHA384",
		    "ECDHE-RSA-AES256-SHA256",
		    "DHE-RSA-AES256-SHA256",
		    "ECDHE-RSA-AES128-SHA256",
		    "DHE-RSA-AES128-SHA256",
		    "HIGH",
		    "!aNULL",
		    "!eNULL",
		    "!EXPORT",
		    "!DES",
		    "!RC4",
		    "!MD5",
		    "!PSK",
		    "!SRP",
		    "!CAMELLIA"
		].join(':'),
		honorCipherOrder: true	
	};
	https.createServer(options, app).listen(config.nodeStartUp.localHttpsPort);
	system.logger.warn('listening on https port: '+config.nodeStartUp.localHttpsPort);
}

if (config.nodeStartUp.protocol==='server') {
	var options = {
		key: fs.readFileSync(packageJson.config.keyFileName),
		cert: fs.readFileSync(packageJson.config.certFileName),
		//ciphers from the iojs -- https://certsimple.com/blog/a-plus-node-js-ssl
		ciphers: [
		    "ECDHE-RSA-AES256-SHA384",
		    "DHE-RSA-AES256-SHA384",
		    "ECDHE-RSA-AES256-SHA256",
		    "DHE-RSA-AES256-SHA256",
		    "ECDHE-RSA-AES128-SHA256",
		    "DHE-RSA-AES128-SHA256",
		    "HIGH",
		    "!aNULL",
		    "!eNULL",
		    "!EXPORT",
		    "!DES",
		    "!RC4",
		    "!MD5",
		    "!PSK",
		    "!SRP",
		    "!CAMELLIA"
		].join(':'),
		honorCipherOrder: true	
	};
	https.createServer(options, app).listen(packageJson.config.processPort);
	system.logger.warn('listening on https port: '+packageJson.config.processPort);
}

//required for debug -- see generator -- no benefit that i noticed
//module.exports = app;
